package middleware

import (
	"net/http"

	ctxUtil "gitlab.wanzhuangkj.com/tush/xpkg/gin/xctx"

	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

func CustomRecoveryWithLogger(logger *zap.Logger) gin.HandlerFunc {
	return gin.CustomRecovery(func(c *gin.Context, err interface{}) {
		logger.Error("Recovered from panic",
			zap.String("path", c.Request.URL.Path),
			zap.String("method", c.Request.Method),
			zap.String("client_ip", c.ClientIP()),
			zap.Any("error", err),
			zap.Stack("stack"),
			ctxUtil.GinTraceIDField(c),
		)
		c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{
			"code":    500,
			"message": "服务器内部错误",
		})
	})
}
