package xtime

import (
	"testing"
	"time"
)

func TestParseExtendedDuration(t *testing.T) {
	type args struct {
		s string
	}
	tests := []struct {
		name    string
		args    args
		want    time.Duration
		wantErr bool
	}{
		{name: "1", args: args{s: "1d"}, want: 24 * time.Hour, wantErr: false},
		{name: "2", args: args{s: "2w"}, want: 2 * 7 * 24 * time.Hour, wantErr: false},
		{name: "3", args: args{s: "3M"}, want: 3 * 30 * 24 * time.Hour, wantErr: false},
		{name: "4", args: args{s: "4y"}, want: 4 * 365 * 24 * time.Hour, wantErr: false},
		{name: "5", args: args{s: "1d2h"}, want: 26 * time.Hour, wantErr: false},
		{name: "6", args: args{s: "1W3D"}, want: (7 + 3) * 24 * time.Hour, wantErr: false},
		{name: "7", args: args{s: "30m"}, want: 30 * time.Minute, wantErr: false},
		{name: "7", args: args{s: "20s"}, want: 20 * time.Second, wantErr: false},
		{name: "7", args: args{s: "20ms"}, want: 20 * time.Millisecond, wantErr: false},
		{name: "8", args: args{s: "1x"}, want: 0, wantErr: true},
		{name: "9", args: args{s: "invalid"}, want: 0, wantErr: true},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := ParseExtendedDuration(tt.args.s)
			if (err != nil) != tt.wantErr {
				t.Errorf("ParseExtendedDuration() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got != tt.want {
				t.Errorf("ParseExtendedDuration() = %v, want %v", got, tt.want)
			}
		})
	}
}
